#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test calculation of CY14 static methods."""

from numpy.testing import assert_almost_equal

from pygmm import ChiouYoungs2014 as CY14


def test_depth_1_0():
    # Value calculated from NGAW2 spreadsheet
    assert_almost_equal(CY14.calc_depth_1_0(600, 'california'), 0.1259, 4)
    assert_almost_equal(CY14.calc_depth_1_0(600, 'japan'), 0.0331, 4)


def test_depth_tor():
    # Value calculated from NGAW2 spreadsheet
    assert_almost_equal(CY14.calc_depth_tor(6, 'SS'), 2.2587685)
    assert_almost_equal(CY14.calc_depth_tor(6, 'RS'), 6.3447262)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
import os

import pytest
import numpy as np

from numpy.testing import assert_allclose

from pygmm.baker_jayaram_2008 import calc_correls, calc_cond_mean_spectrum

fpath = os.path.join(
    os.path.dirname(__file__), 'data', 'baker_jayaram_2008.json')

with open(fpath) as fp:
    data = json.load(fp)


def idfn(case):
    if isinstance(case, dict):
        return 'T_2={period_cond}s'.format(**case)
    else:
        return case


@pytest.mark.parametrize('case', data['cases_correl'], ids=idfn)
def test_calc_correl(case):
    expected = case['correls']
    actual = calc_correls(case['periods'], case['period_cond']).tolist()
    # Points were digitized by hand so need to use larger tolerances
    assert_allclose(actual, expected, atol=0.005, rtol=0.005)


@pytest.mark.parametrize('case', data['cases_cms'], ids=idfn)
@pytest.mark.parametrize('param', ['psas_cms', 'ln_stds_cms'])
def test_calc_cond_spectrum(case, param):
    model = data['model']
    results = calc_cond_mean_spectrum(
        model['periods'],
        np.log(model['psas']),
        model['ln_stds'],
        case['period_cond'],
        np.log(case['psa_cond']), )
    # Need to go from ln_psa to psa
    actual = np.exp(results[0]) if param == 'psas_cms' else results[1]
    expected = case[param]
    assert_allclose(actual, expected, atol=0.0001, rtol=0.0005)




import gzip
import json
import os

import numpy as np
import pytest

from pygmm.model import Scenario
from pygmm import AbrahamsonGregorAddo2016 as AGA16

# Relative tolerance for all tests
RTOL = 2e-2

# Load the tests
fname = os.path.join(os.path.dirname(__file__), 'data', 'aga16_tests.json.gz')
with gzip.open(fname, 'rt') as fp:
    tests = json.load(fp)


def create_model(params):
    s = Scenario(**params)
    m = AGA16(s)
    return m


@pytest.mark.parametrize('test', tests)
@pytest.mark.parametrize(
    'key', ['spec_accels', 'ln_stds', 'pga', 'ln_std_pga'])
def test_spec_accels(test, key):
    m = create_model(test['params'])
    np.testing.assert_allclose(
        getattr(m, key),
        test['results'][key],
        rtol=RTOL,
        err_msg=key
    )



# -*- coding: utf-8 -*-
"""Chiou and Youngs (2014, :cite:`chiou14`) model."""

import logging

import numpy as np

from . import model

__author__ = 'Albert Kottke'


class ChiouYoungs2014(model.GroundMotionModel):
    """Chiou and Youngs (2014, :cite:`chiou14`) model.

    This model was developed for active tectonic regions as part of the
    NGA-West2 effort.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Chiou and Youngs (2014)'
    ABBREV = 'CY14'

    # Reference velocity (m/s)
    V_REF = 1130.

    # Load the coefficients for the model
    COEFF = model.load_data_file('chiou_youngs_2014.csv', 2)

    PERIODS = COEFF['period']

    INDICES_PSA = np.arange(24)
    INDEX_PGA = 24
    INDEX_PGV = 25

    PARAMS = [
        model.NumericParameter('dist_rup', True, 0, 300),
        model.NumericParameter('dist_x', True),
        model.NumericParameter('dist_jb', True),
        model.NumericParameter('mag', True, 3.5, 8.5),
        model.NumericParameter('v_s30', True, 180, 1500),
        model.NumericParameter('depth_tor', False, 0, 20),
        model.NumericParameter('depth_1_0', False),
        model.NumericParameter('dpp_centered', False, default=0),
        model.NumericParameter('dip', True),
        model.CategoricalParameter('mechanism', False, ['U', 'SS', 'NS', 'RS'],
                                   'U'),
        model.CategoricalParameter('on_hanging_wall', False, [True, False],
                                   False),
        model.CategoricalParameter('region', False,
                                   ['california', 'china', 'italy', 'japan'],
                                   'california'),
        model.CategoricalParameter('vs_source', False,
                                   ['measured', 'inferred'], 'measured'),
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model."""
        super().__init__(scenario)
        ln_resp_ref = self._calc_ln_resp_ref()
        self._ln_resp = self._calc_ln_resp_site(ln_resp_ref)
        self._ln_std = self._calc_ln_std(np.exp(ln_resp_ref))

    def _calc_ln_resp_ref(self) -> np.ndarray:
        """Calculate the natural logarithm of the reference response.

        Returns
        -------
        ln_resp_ref : class:`np.array`:
            natural log of the response

        """
        c = self.COEFF
        s = self._scenario

        cosh_mag = np.cosh(2 * max(s.mag - 4.5, 0))
        ln_resp = np.array(c.c_1)

        if s.mechanism == 'RS':
            # Reverse fault term
            ln_resp += (c.c_1a + c.c_1c / cosh_mag)
        elif s.mechanism == 'NS':
            # Normal fault term
            ln_resp += (c.c_1b + c.c_1d / cosh_mag)

        # Magnitude scaling
        ln_resp += c.c_2 * (s.mag - 6)
        ln_resp += (c.c_2 - c.c_3
                    ) / c.c_n * np.log(1 + np.exp(c.c_n * (c.c_m - s.mag)))

        # Top of rupture term relative to model average
        diff_depth_tor = (
            s.depth_tor - self.calc_depth_tor(s.mag, s.mechanism))
        ln_resp += (c.c_7 + c.c_7b / cosh_mag) * diff_depth_tor

        # Dip angle term
        ln_resp += (
            c.c_11 + c.c_11b / cosh_mag) * np.cos(np.radians(s.dip)) ** 2

        # Distance terms
        ln_resp += c.c_4 * np.log(s.dist_rup + c.c_5 * np.cosh(
            c.c_6 * np.maximum(s.mag - c.c_hm, 0)))
        ln_resp += (
            c.c_4a - c.c_4) * np.log(np.sqrt(s.dist_rup ** 2 + c.c_rb ** 2))

        # Regional adjustment
        if s.region in ['japan', 'italy'] and (6 < s.mag < 6.9):
            scale = c.gamma_ji
        elif s.region in ['china']:
            scale = c.gamma_c
        else:
            scale = 1.
        ln_resp += (scale * (c.c_gamma1 + c.c_gamma2 / np.cosh(
            np.maximum(s.mag - c.c_gamma3, 0))) * s.dist_rup)

        # Directivity term
        ln_resp += (c.c_8 * max(1 - max(s.dist_rup - 40, 0) / 30, 0) *
                    min(max(s.mag - 5.5, 0) / 0.8, 1) *
                    np.exp(-c.c_8a * (s.mag - c.c_8b) ** 2) * s.dpp_centered)

        # Hanging wall term
        if s.on_hanging_wall:
            ln_resp += (c.c_9 * np.cos(np.radians(s.dip)) *
                        (c.c_9a + (1 - c.c_9a) * np.tanh(s.dist_x / c.c_9b)) *
                        (1 - np.sqrt(s.dist_jb ** 2 + s.depth_tor ** 2) /
                         (s.dist_rup + 1)))

        return ln_resp

    def _calc_ln_resp_site(self, ln_resp_ref: np.ndarray) -> np.ndarray:
        """Calculate the natural logarithm of the response.

        Parameters
        ----------
        ln_resp_ref : :class:`np.array`
            natural logarithm of the response at the reference site condition
            at each of the periods specified by the model coefficients.

        Returns
        -------
        ln_resp_rsite : class:`np.ndarray`:
            natural log of the response including the site effects

        """
        c = self.COEFF
        s = self._scenario

        if s.region in ['japan']:
            phi_1 = c.phi_1jp
            phi_5 = c.phi_5jp
            phi_6 = c.phi_6jp
        else:
            phi_1 = c.phi_1
            phi_5 = c.phi_5
            phi_6 = c.phi_6

        ln_resp = np.array(ln_resp_ref)
        ln_resp += phi_1 * min(np.log(s.v_s30 / 1130.), 0)

        ln_resp += (c.phi_2 * (
            np.exp(c.phi_3 *
                   (min(s.v_s30, 1130.) - 360.)) - np.exp(c.phi_3 *
                                                          (1130. - 360.))) *
                    np.log((np.exp(ln_resp_ref) + c.phi_4) / c.phi_4))

        diff_depth_1_0 = 1000 * (
            s.depth_1_0 - self.calc_depth_1_0(s.v_s30, s.region))
        ln_resp += phi_5 * (1 - np.exp(-diff_depth_1_0 / phi_6))

        return ln_resp

    def _calc_ln_std(self, resp_ref: np.ndarray) -> np.ndarray:
        """Calculate the logarithmic standard deviation.

        Parameters
        ----------
        ln_resp_ref : :class:`np.array`
            natural logarithm of the response at the reference site condition
            at each of the periods specified by the model coefficients.

        Returns
        -------
        ln_std : class:`np.array`:
            natural log standard deviation

        """
        c = self.COEFF
        s = self._scenario

        if s.region in ['japan']:
            sigma_2 = c.sigma_2jp
        else:
            sigma_2 = c.sigma_2

        clipped_mag = np.clip(s.mag, 5., 6.5) - 5.
        tau = c.tau_1 + (c.tau_2 - c.tau_1) / 1.5 * clipped_mag

        nl_0 = (c.phi_2 * (
            np.exp(c.phi_3 *
                   (min(s.v_s30, 1130.) - 360.)) - np.exp(c.phi_3 *
                                                          (1130. - 360.))) *
                (resp_ref / (resp_ref + c.phi_4)))

        flag_meas = 1 if s.vs_source == 'measured' else 0
        phi_nl = ((c.sigma_1 + (sigma_2 - c.sigma_1) / 1.5 * clipped_mag) *
                  np.sqrt(c.sigma_3 * (1 - flag_meas) + 0.7 * flag_meas + (
                      1 + nl_0) ** 2))

        ln_std = np.sqrt((1 + nl_0) ** 2 * tau ** 2 + phi_nl ** 2)
        return ln_std

    def _check_inputs(self) -> None:
        """Check the inputs."""
        super(ChiouYoungs2014, self)._check_inputs()
        s = self._scenario

        if s.mechanism in ['RS', 'NS']:
            _min, _max = 3.5, 8.0
        else:
            _min, _max = 3.5, 8.5

        if not (_min <= s.mag <= _max):
            logging.warning(
                'Magnitude (%g) exceeds recommended bounds (%g to %g)'
                ' for a %s earthquake!', s.mag, _min, _max, s.mechanism)

        if s.get('depth_tor', None) is None:
            s['depth_tor'] = self.calc_depth_tor(s.mag, s.mechanism)

        if s.get('depth_1_0', None) is None:
            # Calculate depth (m) and convert to (km)
            s['depth_1_0'] = self.calc_depth_1_0(s.v_s30, s.region)

    @staticmethod
    def calc_depth_1_0(v_s30: float, region: str) -> float:
        """Calculate the depth to 1 km/sec (:math:`Z_{1.0}`).

        Parameters
        ----------
        v_s30 : float
            time-averaged shear-wave velocity over the top 30 m
            of the site (:math:`V_{s30}`, m/s).
        region : str
            basin region. Valid options: "california", "japan"

        Returns
        -------
        depth_1_0 : float
            estimated depth to a shear-wave velocity of 1 km/sec (km)

        """
        if region in ['japan']:
            # Japan
            power = 2
            v_ref = 412.39
            slope = -5.23 / power
        else:
            # Global
            power = 4
            v_ref = 570.94
            slope = -7.15 / power

        return np.exp(slope * np.log((v_s30 ** power + v_ref ** power) /
                                     (1360. ** power + v_ref ** power))) / 1000

    @staticmethod
    def calc_depth_tor(mag: float, mechanism: str) -> float:
        """Calculate an estimate of the depth to top of rupture (km).

        Parameters
        ----------
        mag : float
            moment magnitude of the event (:math:`M_w`)
        mechanism : str
            fault mechanism. Valid options: "U", "SS", "NS",
            "RS".

        Returns
        -------
        depth_tor : float
            estimated depth to top of rupture (km)

        """
        if mechanism == 'RS':
            # Reverse and reverse-oblique faulting
            depth_tor = 2.704 - 1.226 * max(mag - 5.849, 0)
        else:
            # Combined strike-slip and normal faulting
            depth_tor = 2.673 - 1.136 * max(mag - 4.970, 0)

        return max(depth_tor, 0) ** 2



# -*- coding: utf-8 -*-
"""Atkinson and Boore (2006, :cite:`atkinson06`) model."""

import numpy as np

from . import model

__author__ = 'Albert Kottke'


class AtkinsonBoore2006(model.GroundMotionModel):
    """Atkinson and Boore (2006, :cite:`atkinson06`) model.

    Developed for the Eastern North America with a reference velocity of 760
    or 2000 m/s.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Atkinson and Boore (2006)'
    ABBREV = 'AB06'

    # Load the coefficients for the model
    COEFF = dict(
        bc=model.load_data_file('atkinson_boore_2006-bc.csv', 2),
        rock=model.load_data_file('atkinson_boore_2006-rock.csv', 2), )

    PERIODS = COEFF['bc']['period']

    COEFF_SITE = model.load_data_file('atkinson_boore_2006-site.csv', 2)
    COEFF_SF = model.load_data_file('atkinson_boore_2006-sf.csv', 2)

    INDEX_PGD = 0
    INDEX_PGV = 1
    INDEX_PGA = 2
    INDICES_PSA = np.arange(3, 27)

    PARAMS = [
        model.NumericParameter('mag', True),
        model.NumericParameter('dist_rup', True),
        model.NumericParameter('v_s30', True)
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model."""
        super(AtkinsonBoore2006, self).__init__(scenario)
        self._ln_resp = self._calc_ln_resp()
        self._ln_std = self._calc_ln_std()

    def _calc_ln_resp(self) -> np.ndarray:
        """Calculate the natural logarithm of the response.

        Returns
        -------
        ln_resp : class:`np.array`:
            natural log of the response
        """
        s = self._scenario
        c = self.COEFF['bc'] if s.v_s30 else self.COEFF['rock']

        # Compute the response at the reference condition
        r0 = 10.0
        r1 = 70.0
        r2 = 140.0

        f0 = np.maximum(np.log10(r0 / s.dist_rup), 0)
        f1 = np.minimum(np.log10(s.dist_rup), np.log10(r1))
        f2 = np.maximum(np.log10(s.dist_rup / r2), 0)

        # Compute the log10 PSA in units of cm/sec/sec
        log10_resp = (c.c_1 + c.c_2 * s.mag + c.c_3 * s.mag ** 2 +
                      (c.c_4 + c.c_5 * s.mag) * f1 +
                      (c.c_6 + c.c_7 * s.mag) * f2 +
                      (c.c_8 + c.c_9 * s.mag) * f0 + c.c_10 * s.dist_rup)

        # Apply stress drop correction
        log10_resp += self._calc_stress_factor()

        if s.v_s30:
            # Compute the site amplification
            pga_bc = (10 ** log10_resp[self.INDEX_PGA])

            log10_site = self._calc_log10_site(pga_bc)

            log10_resp += log10_site

        # Convert from cm/sec/sec to gravity
        log10_resp -= np.log10(980.665)

        ln_resp = np.log(10 ** log10_resp)
        return ln_resp

    def _calc_ln_std(self) -> np.ndarray:
        """Calculate the logarithmic standard deviation.

        Returns
        -------
        ln_std : class:`np.array`:
            natural log standard deviation

        """
        ln_std = np.ones_like(self.PERIODS) * 0.30
        return ln_std

    def _calc_stress_factor(self) -> float:
        """Calculate the stress correction factor proposed by Atkinson and
        Boore (2011) :cite:`atkinson11`.

        Returns
        -------
        log10_stress_factor : class:`np.array`:
            log base 10 of the stress factor
        """
        s = self._scenario
        c = self.COEFF_SF

        stress_drop = 10. ** (3.45 - 0.2 * s.mag)
        v1 = c.delta + 0.05
        v2 = (0.05 + c.delta * np.maximum(s.mag - c.m_1, 0) / (c.m_h - c.m_1))

        log10_stress_factor = (np.minimum(2., stress_drop / 140.) *
                               np.minimum(v1, v2))

        return np.interp(self.PERIODS, c.period, log10_stress_factor)

    def _calc_log10_site(self, pga_bc: float) -> np.ndarray:
        """Calculate the log10 of the site amplification.

        Parameters
        ----------
        pga_bc : float
            peak ground acceleration (PGA, g) at the B/C boundary.

        Returns
        -------
        log_10_site : :class:`np.ndarray`
            log base 10 of the  site amplification.
        """
        s = self._scenario
        c = self.COEFF_SITE
        VS_1 = 180.
        VS_2 = 300.
        VS_REF = 760.

        if s.v_s30 <= VS_1:
            b_nl = c.b_1
        elif VS_1 < s.v_s30 <= VS_2:
            b_nl = (
                (c.b_1 - c.b_2) * np.log(s.v_s30 / VS_2) / np.log(VS_1 / VS_2))
        elif VS_2 < s.v_s30 <= VS_REF:
            b_nl = (c.b_2 * np.log(s.v_s30 / VS_REF) / np.log(VS_2 / VS_REF))
        else:
            # Vs30 > VS_REF
            b_nl = 0

        pga_bc = max(pga_bc, 60.)

        log10_site = np.log10(
            np.exp(c.b_lin * np.log(s.v_s30 / VS_REF) + b_nl * np.log(pga_bc /
                                                                      100.)))
        return np.interp(self.PERIODS, c.period, log10_site)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test model interface using C03 model."""

from pygmm.model import Parameter, NumericParameter, CategoricalParameter

import pytest


def test_parameter():
    p = Parameter('test')
    p.check(None)


@pytest.fixture
def numeric_parameter():
    return NumericParameter('test', required=True, min_=0, max_=10)


@pytest.fixture
def categorical_parameter():
    return CategoricalParameter(
        'test', required=True, options=['spam', 'eggs'])


# See https://github.com/pytest-dev/pytest/issues/349
@pytest.fixture(params=['numeric_parameter', 'categorical_parameter'])
def param(request):
    return request.getfixturevalue(request.param)


def test_required(param):
    with pytest.raises(ValueError):
        param.check(None)



# -*- coding: utf-8 -*-
"""pyGMM: Ground motion models implemented in Python."""

import logging
import pkg_resources

from .model import Scenario
from .abrahamson_silva_1996 import AbrahamsonSilva1996
from .abrahamson_gregor_addo_2016 import AbrahamsonGregorAddo2016
from .abrahamson_silva_kamai_2014 import AbrahamsonSilvaKamai2014
from .atkinson_boore_2006 import AtkinsonBoore2006
from .akkar_sandikkaya_bommer_2014 import AkkarSandikkayaBommer2014
from .boore_stewart_seyhan_atkinson_2014 import BooreStewartSeyhanAtkinson2014
from .campbell_2003 import Campbell2003
from .campbell_bozorgnia_2014 import CampbellBozorgnia2014
from .chiou_youngs_2014 import ChiouYoungs2014
from .derras_bard_cotton_2014 import DerrasBardCotton2014
from .idriss_2014 import Idriss2014
from .pezeshk_zandieh_tavakoli_2011 import PezeshkZandiehTavakoli2011
from .tavakoli_pezeshk_2005 import TavakoliPezeshk05

__all__ = [
    'Scenario',
    'AbrahamsonSilvaKamai2014',
    'AbrahamsonGregorAddo2016',
    'AkkarSandikkayaBommer2014',
    'AtkinsonBoore2006',
    'BooreStewartSeyhanAtkinson2014',
    'Campbell2003',
    'CampbellBozorgnia2014',
    'ChiouYoungs2014',
    'DerrasBardCotton2014',
    'Idriss2014',
    'PezeshkZandiehTavakoli2011',
    'TavakoliPezeshk05',
]

__author__ = 'Albert Kottke'
__copyright__ = 'Copyright 2016 Albert Kottke'
__license__ = 'MIT'
__title__ = 'pygmm'
__version__ = pkg_resources.get_distribution('pygmm').version

# Set default logging handler to avoid "No handler found" warnings.
try:  # Python 2.7+
    from logging import NullHandler
except ImportError:

    class NullHandler(logging.Handler):
        def emit(self, record):
            pass


logging.getLogger(__name__).addHandler(NullHandler())

models = [
    AbrahamsonSilvaKamai2014,
    AkkarSandikkayaBommer2014,
    AtkinsonBoore2006,
    BooreStewartSeyhanAtkinson2014,
    Campbell2003,
    CampbellBozorgnia2014,
    ChiouYoungs2014,
    DerrasBardCotton2014,
    Idriss2014,
    PezeshkZandiehTavakoli2011,
    TavakoliPezeshk05,
]



#!/usr/bin/python
# -*- coding: utf-8 -*-

import gzip
import itertools
import json
import os

import numpy as np
import pytest

import pygmm

models = [
    pygmm.AbrahamsonSilvaKamai2014,
    pygmm.BooreStewartSeyhanAtkinson2014,
    pygmm.CampbellBozorgnia2014,
    pygmm.ChiouYoungs2014,
    pygmm.Idriss2014,
]

# Relative tolerance for all tests
RTOL = 1e-2

# Load the tests
fname = os.path.join(os.path.dirname(__file__), 'data', 'ngaw2_tests.json.gz')
with gzip.open(fname, 'rt') as fp:
    tests = json.load(fp)

testdata = [(m, t['params'], t['results'][m.ABBREV])
            for m, t in itertools.product(models, tests)]


@pytest.mark.parametrize('model,params,expected', testdata)
def test_ln_stds(model, params, expected):
    m = model(pygmm.model.Scenario(**params))
    np.testing.assert_allclose(
        m.interp_ln_stds(expected['periods']),
        expected['ln_stds'],
        rtol=RTOL,
        err_msg='Logarithmic standard deviations')


@pytest.mark.parametrize('model,params,expected', testdata)
def test_spec_accels(model, params, expected):
    m = model(pygmm.model.Scenario(**params))
    np.testing.assert_allclose(
        m.interp_spec_accels(expected['periods']),
        expected['spec_accels'],
        rtol=RTOL,
        err_msg='Spectral accelerations')


@pytest.mark.parametrize('model,params,expected', testdata)
@pytest.mark.parametrize('key', ['pga', 'ln_std_pga', 'pgv', 'ln_std_pgv'])
def test_im_values(model, params, expected, key):
    if expected.get(key, None) is None:
        return

    m = model(pygmm.model.Scenario(**params))

    try:
        value = getattr(m, key)
    except NotImplementedError:
        return

    if value is None:
        return

    np.testing.assert_allclose(
        getattr(m, key),
        expected[key],
        rtol=RTOL, )



"""Boore, Stewart, Seyhan, and Atkinson (2014) ground motion model."""

import logging

import numpy as np

from . import model
from .chiou_youngs_2014 import ChiouYoungs2014 as CY14
from .types import ArrayLike

__author__ = 'Albert Kottke'


class BooreStewartSeyhanAtkinson2014(model.GroundMotionModel):
    """Boore, Stewart, Seyhan, and Atkinson (2014, :cite:`boore14`) model.

    This model was developed for active tectonic regions as part of the
    NGA-West2 effort.

    The BSSA14 model defines the following distance attenuation models:

        +--------------+-------------------------------+
        | Name         | Description                   |
        +--------------+-------------------------------+
        | global       | Global; California and Taiwan |
        +--------------+-------------------------------+
        | china_turkey | China and Turkey              |
        +--------------+-------------------------------+
        | italy_japan  | Italy and Japan               |
        +--------------+-------------------------------+

    and the following basin region models:

        +--------+---------------------+
        | Name   | Description         |
        +========+=====================+
        | global | Global / California |
        +--------+---------------------+
        | japan  | Japan               |
        +--------+---------------------+

    These are simplified into one regional parameter with the following
    possibilities:

        +-------------+--------------+------------+
        | Region      | Attenuation  | Basin      |
        +=============+==============+============+
        | global      | global       | global     |
        +-------------+--------------+------------+
        | california  | global       | global     |
        +-------------+--------------+------------+
        | china       | china_turkey | global     |
        +-------------+--------------+------------+
        | italy       | italy_japan  | global     |
        +-------------+--------------+------------+
        | japan       | italy_japan  | japan      |
        +-------------+--------------+------------+
        | new zealand | italy_japan  | global     |
        +-------------+--------------+------------+
        | taiwan      | global       | global     |
        +-------------+--------------+------------+
        | turkey      | china_turkey | global     |
        +-------------+--------------+------------+

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """
    NAME = 'Boore, Stewart, Seyhan, and Atkinson (2014)'
    ABBREV = 'BSSA14'

    # Reference shear-wave velocity in m/sec
    V_REF = 760.

    # Load the coefficients for the model
    COEFF = model.load_data_file('boore_stewart_seyhan_atkinson-2014.csv', 2)
    PERIODS = COEFF['period']

    INDEX_PGV = 0
    INDEX_PGA = 1
    INDICES_PSA = np.arange(2, 107)

    LIMITS = dict(
        mag=(3.0, 8.5),
        dist_jb=(0., 300.),
        v_s30=(150., 1500.), )

    PARAMS = [
        model.NumericParameter('mag', True, 3, 8.5),
        model.NumericParameter('depth_1_0', False),
        model.NumericParameter('dist_jb', True, None, 300.),
        model.NumericParameter('v_s30', True, 150., 1500.),
        model.CategoricalParameter('mechanism', False, ['U', 'SS', 'NS', 'RS'],
                                   'U'),
        model.CategoricalParameter('region', False, [
            'global', 'california', 'china', 'italy', 'japan', 'new_zealand',
            'taiwan', 'turkey'
        ], 'global'),
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model.

        Args:
            scenario (:class:`pygmm.model.Scenario`): earthquake scenario.
        """
        super(BooreStewartSeyhanAtkinson2014, self).__init__(scenario)
        pga_ref = np.exp(self._calc_ln_resp(np.nan)[self.INDEX_PGA])
        self._ln_resp = self._calc_ln_resp(pga_ref)
        self._ln_std = self._calc_ln_std()

    def _check_inputs(self) -> None:
        """Check the inputs."""
        super(BooreStewartSeyhanAtkinson2014, self)._check_inputs()
        s = self._scenario
        # Mechanism specific limits
        if s.mechanism == 'SS':
            _min, _max = 3., 8.5
            if not (_min <= s.mag <= _max):
                logging.warning(
                    'Magnitude (%g) exceeds recommended bounds (%g to %g)'
                    ' for a strike-slip earthquake!', s.mag, _min, _max)
        elif s.mechanism == 'NS':
            _min, _max = 3., 7.0
            if not (_min <= s.mag <= _max):
                logging.warning(
                    'Magnitude (%g) exceeds recommended bounds (%g to %g)'
                    ' for a normal-slip earthquake!', s.mag, _min, _max)

    def _calc_ln_resp(self, pga_ref: ArrayLike) -> np.ndarray:
        """Calculate the natural logarithm of the response.

        Parameters
        ----------
        pga_ref : float
            peak ground acceleration (g) at the reference
            condition. If :class:`np.nan`, then no site term is applied.

        Returns
        -------
        ln_resp : class:`np.array`:
            natural log of the response

        """
        s = self._scenario
        c = self.COEFF

        # Compute the event term
        ########################
        if s.mechanism == 'SS':
            event = np.array(c.e_1)
        elif s.mechanism == 'NS':
            event = np.array(c.e_2)
        elif s.mechanism == 'RS':
            event = np.array(c.e_3)
        else:
            # Unspecified
            event = np.array(c.e_0)

        mask = s.mag <= c.M_h
        event[mask] += (c.e_4 * (s.mag - c.M_h) + c.e_5 *
                        (s.mag - c.M_h) ** 2)[mask]
        event[~mask] += (c.e_6 * (s.mag - c.M_h))[~mask]

        # Compute the distance terms
        ############################
        if s.region in ['china', 'turkey']:
            dc_3 = c.dc_3ct
        elif s.region in ['italy', 'japan']:
            dc_3 = c.dc_3ij
        else:
            # s.region in 'global', 'california', 'new_zealand', 'taiwan'
            dc_3 = c.dc_3global

        dist = np.sqrt(s.dist_jb ** 2 + c.h ** 2)
        path = (
            (c.c_1 + c.c_2 * (s.mag - c.M_ref)) *
            np.log(dist / c.R_ref) +
            (c.c_3 + dc_3) * (dist - c.R_ref)
        )

        if np.isnan(pga_ref):
            # Reference condition. No site effect
            site = 0
        else:
            # Compute the site term
            f_lin = c.c * np.log(np.minimum(s.v_s30, c.V_c) / c.V_ref)

            # Add the nonlinearity to the site term
            f_2 = c.f_4 * (
                np.exp(c.f_5 *
                       (min(s.v_s30, 760) - 360.)) - np.exp(c.f_5 *
                                                            (760. - 360.)))
            f_nl = c.f_1 + f_2 * np.log((pga_ref + c.f_3) / c.f_3)

            # Add the basin effect to the site term
            F_dz1 = np.zeros_like(c.period)

            # Compute the average from the Chiou and Youngs (2014)
            # model convert from m to km.
            ln_mz1 = np.log(CY14.calc_depth_1_0(s.v_s30, s.region))

            if s.get('depth_1_0', None) is not None:
                delta_depth_1_0 = s.depth_1_0 - np.exp(ln_mz1)
            else:
                delta_depth_1_0 = 0.

            mask = (c.period >= 0.65)
            F_dz1[mask] = np.minimum(c.f_6 * delta_depth_1_0, c.f_7)[mask]

            site = f_lin + f_nl + F_dz1

        ln_resp = event + path + site
        return ln_resp

    def _calc_ln_std(self) -> np.ndarray:
        """Calculate the logarithmic standard deviation.

        Returns
        -------
        ln_std : class:`np.array`:
            natural log standard deviation

        """
        c = self.COEFF
        s = self._scenario

        # Uncertainty model
        tau = c.tau_1 + (c.tau_2 - c.tau_1) * \
                        (np.clip(s.mag, 4.5, 5.5) - 4.5)
        phi = c.phi_1 + (c.phi_2 - c.phi_1) * \
                        (np.clip(s.mag, 4.5, 5.5) - 4.5)

        # Modify phi for Vs30
        phi -= c.dphi_V * np.clip(
            np.log(c.V_2 / s.v_s30) / np.log(c.V_2 / c.V_1), 0, 1)

        # Modify phi for R
        phi += c.dphi_R * np.clip(
            # Maximum added for zero distance caes
            np.log(np.maximum(s.dist_jb, 0.1) / c.R_1) /
            np.log(c.R_2 / c.R_1),
            0, 1
        )


        ln_std = np.sqrt(phi ** 2 + tau ** 2)
        return ln_std



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test calculation of CB14 static methods."""

from numpy.testing import assert_almost_equal

from pygmm import CampbellBozorgnia2014 as CB14


def test_depth_2_5():
    # Value calculated from NGAW2 spreadsheet
    assert_almost_equal(CB14.calc_depth_2_5(600, 'japan', None), 0.1844427)
    assert_almost_equal(
        CB14.calc_depth_2_5(600, 'california', None), 0.7952589)



#!/usr/bin/env python
# encoding: utf-8
"""Gülerce and Abrahamson (2011, :cite:`gulerce11`) V/H model."""

from __future__ import division

import numpy as np

from . import model
from .abrahamson_silva_kamai_2014 import AbrahamsonSilvaKamai2014 as ASK14

__author__ = 'Albert Kottke'


class GulerceAbrahamson2011(model.Model):
    """Gülerce and Abrahamson (2011, :cite:`gulerce11`) V/H model.

    This model was developed for active tectonic regions.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Gülerce & Abrahamson (2011)'
    ABBREV = 'GA11'

    # Reference velocity (m/s)
    V_REF = 1100.

    # Load the coefficients for the model
    COEFF = model.load_data_file(
        'gulerce_abrahamson_2011-coefficients.csv', 2)
    PERIODS = COEFF['period']

    INDEX_PGA = 0
    INDEX_PGV = 1
    INDICES_PSA = 2 + np.arange(27)

    PARAMS = [
        model.NumericParameter('dist_rup', True, None, 200),
        model.NumericParameter('mag', True, 5, 8.5),
        model.NumericParameter('v_s30', True, 450, 1200),
        model.NumericParameter('pga_ref', False, 0.001, 10),
        model.CategoricalParameter('mechanism', True, ['SS', 'NS', 'RS']),
    ]

    def __init__(self, *args, **kwargs):
        """Initialize the model."""
        super(GulerceAbrahamson2011, self).__init__(*args, **kwargs)

        if 'pga_ref' not in self._scenario:
            s = model.Scenario(self._scenario)
            s['v_s30'] = self.V_REF
            m = ASK14(s)
            self._scenario['pga_ref'] = m.pga

        self._ln_ratio = self._calc_ln_ratio()
        self._ln_std = self._calc_ln_std()


    def _calc_ln_ratio(self):
        s = self._scenario
        c = self.COEFF

        dist = np.sqrt(s.dist_rup ** 2 + c.c4 ** 2)

        # Magnitude and distance scaling
        f1 = (
            c.a1 + c.a8 (8.5 - s.mag) ** 2 +
            (c.a2 + c.a3 * (s.mag - c.c1)) * np.log(dist)
        )
        mask = (s.mag < c.c1)
        f1[mask] += c.a4 * (s.mag - c.c1)
        f1[~mask] += c.a5 * (s.mag - c.c1)

        # Site model
        v_1 = np.select(
            [
                self.PERIODS < 0,
                self.PERIODS <= 0.50,
                self.PERIODS <= 1.00,
                self.PERIODS <= 2.00,
                self.PERIODS <= 10.0,
            ],
            [
                862,
                1500.,
                np.exp(8 - 0.795 * np.log(self.PERIODS / 0.21)),
                np.exp(6.76 - 0.297 * np.log(self.PERIODS)),
                700,
            ]
        )
        v_s30_lim = np.minimum(s.v_s30, v_1)

        f5 = c.a10 * np.log(v_s30_lim / c.v_lin)
        mask = v_s30_lim < c.v_lin
        f5[mask] -= (
            -c.b * np.log(s.pga_ref + c.c) +
            c.b * np.log(s.pga_ref + c * (v_s30_lim / c.v_lin) ** c.n)
        )
        f5[~mask] -= (
            c.b * c.n * np.log(v_s30_lim / c.v_lin)
        )

        ln_ratio = f1 + f5
        if s.mechanism == 'RS':
            ln_ratio += c.a6
        elif s.mechanism == 'NS':
            ln_ratio += c.a7

        return ln_ratio

    def _calc_ln_std_within(self):
        c = self.COEFF
        mag = self._scenario.mag
        return np.clip(
            c.s1 + 0.5 * (c.s2 - c.s1) * (mag - 5),
            c.s1, c.s2
        )

    def _calc_ln_std_between(self):
        c = self.COEFF
        mag = self._scenario.mag
        return np.clip(
            c.s3 + 0.5 * (c.s4 - c.s3) * (mag - 5),
            c.s3, c.s4
        )

    def _calc_ln_std(self):
        return np.sqrt(
            self._calc_ln_std_between() ** 2 +
            self._calc_ln_std_within() ** 2
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Create test cases for the NGA West2 models."""

import gzip
import json
import os

import xlwings as xw


def iter_parameters(parameters):
    max_len = max(len(p) for p in parameters.values())

    for i in range(max_len):
        yield {k: v[i % len(v)] for k, v in parameters.items()}


def reformat_params(params):
    """Transform from pygmm input to spreadsheet input."""
    if 'mechanism' in params:
        # Replace mechanism with flags
        mech = params.pop('mechanism')
        if mech == 'SS':
            params['flag_u'] = 0
            params['flag_rv'] = 0
            params['flag_nm'] = 0
        elif mech == 'NS':
            params['flag_u'] = 0
            params['flag_rv'] = 0
            params['flag_nm'] = 1
        elif mech == 'RS':
            params['flag_u'] = 0
            params['flag_rv'] = 1
            params['flag_nm'] = 0
        elif mech == 'U':
            params['flag_u'] = 1
            params['flag_rv'] = 0
            params['flag_nm'] = 0

    try:
        on_hanging_wall = params.pop('on_hanging_wall')
        params['flag_hw'] = 1 if on_hanging_wall else 0
    except KeyError:
        pass

    try:
        params['flag_meas'] = params.pop('vs_source')
    except KeyError:
        pass

    try:
        params['region'] = params['region'].replace('_', ' ').title()
    except KeyError:
        pass

    # Replace None with 999
    for key in [
            'depth_tor', 'depth_hyp', 'depth_1_0', 'depth_2_5', 'width',
            'dist_y0'
    ]:
        if key in params and params[key] is None:
            params[key] = 999

    if 'dist_rup' not in params and 'dist_jb' in params:
        params['dist_rup'] = params['dist_jb']

    try:
        # No supported to changing the DPP
        params.pop('dpp_centered')
    except KeyError:
        pass

    return params


def load_params(wb, **params):
    params = reformat_params(dict(params))
    for k, v in params.items():
        print('Setting:', k, CELLS[k], v)
        xw.Range('Main', CELLS[k], wkb=wb).value = v

    # Force a re-calculation. This is the same as Ctrl + Alt + F9
    wb.xl_app.CalculateFull()


def get_results(wb, abbrev):
    sheetname = abbrev
    result_columns = RESULT_COLUMNS[abbrev]
    # Collect response spectrum
    d = {}
    for key, rc in zip(['periods', 'spec_accels', 'ln_stds'], result_columns):
        d[key] = xw.Range(
            sheetname, '{col}6:{col}26'.format(col=rc), wkb=wb).value

    # Collect the PGA and PGV
    for key, row in zip(['pga', 'pgv'], [28, 29]):
        values = xw.Range(
            sheetname,
            '{col0:}{row:d}:{col1:}{row:d}'.format(
                col0=result_columns[1], col1=result_columns[2], row=row),
            wkb=wb).value
        for subkey, v in zip(['', '_ln_std'], values):
            try:
                float(v)
            except ValueError:
                v = None
            d[key + subkey] = v

    return d


wb_fname = os.path.abspath(
    os.path.join(
        os.path.dirname(__file__),
        'NGAW2_GMPE_Spreadsheets_v5.7_041415_Protected.xlsm'))

# Parameters
params_site = {
    'v_s30': [300, 450, 760, 1000],
    'depth_1_0': [0.4, 0.3, 0.04, 0.005],
    'depth_2_5': [1.8, 1.1, 0.4, 0.3],
    'vs_source': ['measured', 'inferred'],
    'region': [
        'global',
        'california',
        'china',
        'japan',
        'italy',
        'new_zealand',
        'turkey',
        'taiwan',
    ]
}

params_fault = [
    {
        'mag': 6.,
        'dist_rup': 3.16,
        'dist_jb': 1.,
        'dist_x': 1.,
        'dist_y0': None,
        'mechanism': 'SS',
        'on_hanging_wall': False,
        'dip': 90,
        'depth_tor': 0,
        'depth_hyp': 8,
        'width': 10,
    },
    {
        'mag': 7.1,
        'dist_rup': 10,
        'dist_jb': 2,
        'dist_x': 16,
        'dist_y0': None,
        'mechanism': 'RS',
        'on_hanging_wall': True,
        'dip': 50,
        'depth_tor': 1,
        'depth_hyp': 8,
        'width': 15,
    },
    {
        'mag': 4.5,
        'dist_rup': 10,
        'dist_jb': 8.,
        'dist_x': -8,
        'dist_y0': None,
        'mechanism': 'RS',
        'on_hanging_wall': False,
        'dip': 60,
        'depth_tor': 4,
        'depth_hyp': 5,
        'width': 2,
    },
    {
        'mag': 6.0,
        'dist_rup': 5,
        'dist_jb': 0.,
        'dist_x': 0.1,
        'dist_y0': None,
        'mechanism': 'NS',
        'on_hanging_wall': True,
        'dip': 60,
        'depth_tor': 4,
        'depth_hyp': 5,
        'width': 8,
    },
    {
        'mag': 6.5,
        'dist_rup': 100,
        'dist_jb': 100,
        'dist_x': 60,
        'dist_y0': 70,
        'mechanism': 'U',
        'on_hanging_wall': False,
        'dip': 90,
        'depth_tor': 2,
        'depth_hyp': 7,
        'width': 15,
    },
]

CELLS = dict(
    mag='B24',
    dist_rup='B27',
    dist_jb='B30',
    dist_x='B33',
    dist_y0='B36',
    v_s30='B39',
    flag_u='B42',
    flag_rv='B45',
    flag_nm='B48',
    flag_hw='B51',
    dip='B54',
    depth_tor='B57',
    depth_hyp='B60',
    depth_1_0='B63',
    depth_2_5='B66',
    width='B69',
    flag_meas='B72',
    flag_as='B75',
    region='B78',
    dpp_centered='B83',
    depth_bor='B89', )

RESULT_COLUMNS = dict(
    ASK14='DEF',
    BSSA14='DST',
    CB14='DYZ',
    CY14='DUV',
    I14='DLM', )

abbreviations = ['ASK14', 'BSSA14', 'CB14', 'CY14', 'I14']

# Join the parameters of the site and fault
parameters = dict(params_site)
for pf in params_fault:
    for k, v in pf.items():
        if k not in parameters:
            parameters[k] = list()
        parameters[k].append(v)

tests = []
for p in iter_parameters(parameters):
    wb = xw.Workbook(wb_fname)
    load_params(wb, **p)

    results = {abb: get_results(wb, abb) for abb in abbreviations}

    if p['region'] == 'taiwan':
        # Load ASK14 specific values.
        load_params(wb, region='TWA')
        results['ASK14'] = get_results(wb, 'ASK14')

    tests.append({'params': p, 'results': results})

fname = '../tests/data/ngaw2_tests.json.gz'
with gzip.open(fname, 'wt') as fp:
    json.dump(tests, fp, indent=4)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Create test cases for the DBC13 model."""

import gzip
import json
import os

import xlwings as xw

params = {
    'dist': [5, 40, 200],
    'mag': [4, 6, 7],
    'mechanism': ['NS', 'RS', 'SS'],
    'v_s30': [200, 400, 750, 1200],
}


def iter_parameters(parameters):
    max_len = max(len(p) for p in parameters.values())

    for i in range(max_len):
        yield {k: v[i % len(v)] for k, v in parameters.items()}


def reformat_params(params):
    """Translate to spreadsheet values."""
    params['mechanism'] = ['NS', 'RS', 'SS'].index(params['mechanism'])
    return params


def load_params(wb, **params):
    cell_map = {
        'mag': 'B1',
        'v_s30': 'B6',
        'mechanism': 'B5',
    }
    params = reformat_params(dict(params))
    for k, v in params.items():
        if k == 'dist':
            # Update all distances
            for c in ['B2', 'B3', 'B4']:
                xw.Range('Input-Output', c, wkb=wb).value = v
        else:
            xw.Range('Input-Output', cell_map[k], wkb=wb).value = v

    # Force a re-calculation. This is the same as Ctrl + Alt + F9
    wb.xl_app.CalculateFull()


def get_results(wb):
    results = []
    for col, dist in zip('GHI', ['jb', 'epi', 'hyp']):
        d = {}
        # Collect PGA and PGV
        for key, row in zip(['pga', 'pgv'], [3, 4]):
            d[key] = \
                xw.Range('Input-Output', '%s%d' % (col, row), wkb=wb).value

        # Collect response spectrum
        for key, c in zip(['periods', 'spec_accels'], ['F', col]):
            d[key] = \
                xw.Range('Input-Output', '%s5:%s66' % (c, c), wkb=wb).value

        results.append(('dist_' + dist, d))

    return results


wb_fname = os.path.abspath('ASB14.xlsx')
tests = []
for p in iter_parameters(params):
    wb = xw.Workbook(wb_fname)
    load_params(wb, **p)
    tests.append({'params': p, 'results': get_results(wb)})

fname = '../tests/data/asb14_tests.json.gz'
with gzip.open(fname, 'wt') as fp:
    json.dump(tests, fp, indent=4)



# -*- coding: utf-8 -*-
"""Model for the Campbell (2003) ground motion model."""

import numpy as np

from . import model

__author__ = 'Albert Kottke'


class Campbell2003(model.GroundMotionModel):
    """Campbell (2003, :cite:`campbell03`) model.

    This model was developed for the Eastern US.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario


    """

    NAME = 'Campbell (2003)'
    ABBREV = 'C03'

    # Reference velocity (m/sec)
    V_REF = 2800.

    COEFF = model.load_data_file('campbell_2003.csv', 1)
    PERIODS = COEFF['period']

    INDICES_PSA = np.arange(16)

    PARAMS = [
        model.NumericParameter('mag', True, 5.0, 8.2),
        model.NumericParameter('dist_rup', True, None, 1000.),
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model."""
        super().__init__(scenario)
        self._ln_resp = self._calc_ln_resp()
        self._ln_std = self._calc_ln_std()

    def _calc_ln_resp(self) -> np.ndarray:
        """Calculate the natural logarithm of the response.

        Returns
        -------
        ln_resp : class:`np.array`:
            natural log of the response

        """
        c = self.COEFF
        s = self._scenario

        f_1 = c.c_2 * s.mag + c.c_3 * (8.5 - s.mag) ** 2
        # Distance scaling
        f_2 = (c.c_4 * np.log(
            np.sqrt(s.dist_rup ** 2 + (c.c_7 * np.exp(c.c_8 * s.mag)) ** 2)) +
               (c.c_5 + c.c_6 * s.mag) * s.dist_rup)
        # Geometric attenuation
        r_1 = 70.0
        r_2 = 130.0
        if s.dist_rup <= r_1:
            f_3 = 0.
        else:
            f_3 = c.c_9 * (np.log(s.dist_rup) - np.log(r_1))

            if r_2 < s.dist_rup:
                f_3 += c.c_10 * (np.log(s.dist_rup) - np.log(r_2))

        # Compute the ground motion
        ln_resp = c.c_1 + f_1 + f_2 + f_3

        return ln_resp

    def _calc_ln_std(self) -> np.ndarray:
        """Calculate the logarithmic standard deviation.

        Returns
        -------
        ln_std : class:`np.array`:
            natural log standard deviation

        """
        c = self.COEFF
        s = self._scenario

        if s.mag < 7.16:
            ln_std = c.c_11 + c.c_12 * s.mag
        else:
            ln_std = c.c_13

        return ln_std



#!/usr/bin/env python3
# encoding: utf-8
"""Hermkes, Kuehn, Riggelsen (2014, :cite:`hermkes14`) model."""

import pathlib
import logging

import numpy as np
from scipy.interpolate import NearestNDInterpolator

from . import model

__author__ = 'Albert Kottke'

fname_data = pathlib.Path(__file__).parent.joinpath(
    'data', 'hermkes_kuehn_riggelsen_2014.npz')

if not fname_data.exists():
    # Download the model data if not found.
    import urllib.request
    import shutil

    url = ('https://www.dropbox.com/s/1tu9ss1s3inctej/'
           'hermkes_kuehn_riggelsen_2014.npz?dl=0')

    try:
        with urllib.request.urlopen(url) as req, fname_data.open('wb') as fp:
            shutil.copy(req, fp)
    except urllib.request.URLError:
        logging.critical(
            'Hermkes, Kuehn, and Riggelsen (2013) model data required, '
            'which cannot be downloaded. Download the file from %s'
            'to this location: %s', url, fname_data)

INTERPOLATOR = None


class HermkesKuehnRiggelsen2014(model.GroundMotionModel):
    """Hermkes, Kuehn, Riggelsen (2014, :cite:`hermkes14`) model.

    Only the *GPSELinCorr* model is implemented. This model must be imported
    directly by::

        from pygmm.hermkes_kuehn_riggelsen_2014 import
        HermkesKuehnRiggelsen2014

    This is to due to the large file size of the model data, which takes
    time to load.

    Note that this model was developed using a Bayesian non-parametric
    method, which means it is should only be used over the data range
    used to develop the model. See the paper for more details.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Hermkes, Kuehn, Riggelsen (2014)'
    ABBREV = 'HKR14'

    # Reference velocity (m/sec)
    V_REF = None

    PERIODS = np.array([-1, 0.01, 0.1, 0.5, 1.0, 4.0])
    INDICES_PSA = np.arange(1, 6)
    INDEX_PGA = 1
    INDEX_PGV = 0
    PARAMS = [
        model.NumericParameter('depth_hyp', False, 0, 40, 15),
        model.NumericParameter('dist_jb', False, 0, 200),
        model.NumericParameter('mag', True, 4, 8),
        model.NumericParameter('v_s30', True, 100, 1200),
        model.CategoricalParameter('mechanism', True, ['SS', 'NS', 'RS']),
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model."""
        super().__init__(scenario)

        s = self._scenario

        flag_rs = flag_ss = flag_ns = 0
        if s.mechanism == 'SS':
            flag_ss = 1
        elif s.mechanism == 'NS':
            flag_ns = 1
        elif s.mechanism == 'RS':
            flag_rs = 1

        event = (s.mag, s.depth_hyp, flag_rs, flag_ss, flag_ns, s.dist_jb,
                 s.v_s30)

        global INTERPOLATOR
        if INTERPOLATOR is None:
            with np.load(fname_data) as data:
                INTERPOLATOR = NearestNDInterpolator(data['events'],
                                                     data['predictions'])
        prediction = INTERPOLATOR(event)
        self._ln_resp = prediction[0::2]
        self._ln_std = np.sqrt(prediction[1::2])



"""Reference case from AbrahamsonSilva1996 based on spreadsheet by N.
Gregor."""

import json
import os

import pytest

from numpy.testing import assert_allclose

import pygmm

fpath = os.path.join(os.path.dirname(__file__), 'data', 'as96.json')
with open(fpath) as fp:
    data = json.load(fp)


@pytest.mark.parametrize('case', data)
def test_calc_duration(case):
    s = pygmm.Scenario(**case['scenario'])
    m = pygmm.AbrahamsonSilva1996(s)

    nias = case['nias']
    stds = case['stds']

    actual = m.interp(nias, stds)

    assert_allclose(actual, case['durs'], atol=0.0001, rtol=0.01)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test model interface using C03 model."""

from numpy.testing import assert_array_equal, assert_allclose

from pygmm.model import Scenario
from pygmm import Campbell2003 as C03

import pytest


@pytest.fixture
def model():
    return C03(Scenario(mag=6.5, dist_rup=20))


def test_ln_std(model):
    assert_array_equal(model._ln_std, model.ln_stds)


def test_scenario():
    s = Scenario(mag=6, dist_jb=20)
    assert_allclose(s.mag, s['mag'])
    assert_allclose(s.dist_jb, s['dist_jb'])


@pytest.mark.parametrize(
    'attr', ['pga', 'ln_std_pga', 'pgv', 'ln_std_pgv', 'pgd', 'ln_std_pgd'])
def test_pga(model, attr):
    with pytest.raises(NotImplementedError):
        getattr(model, attr)



import json
import os

import numpy as np
import pytest

from numpy.testing import assert_allclose

from pygmm.kishida_2017 import calc_cond_mean_spectrum_vector

fpath = os.path.join(os.path.dirname(__file__), 'data', 'kishida_2017.json')
with open(fpath) as fp:
    data = json.load(fp)


def idfn(case):
    if isinstance(case, dict):
        return str(case['periods_cond'])


@pytest.mark.parametrize('case', data['cases'], ids=idfn)
@pytest.mark.parametrize('param', ['psas_cmsv', 'ln_stds_cmsv'])
def test_calc_cond_mean_spectrum_vector(case, param):
    model = data['model']
    ln_psas_cond = np.ma.array(
        np.log(model['psas_target']),
        mask=(~np.in1d(model['periods'], case['periods_cond'])))
    results = calc_cond_mean_spectrum_vector(model['periods'],
                                             np.log(model['psas']),
                                             model['ln_stds'], ln_psas_cond)
    # Need to go from ln_psa to psa
    actual = np.exp(results[0]) if param == 'psas_cmsv' else results[1]
    expected = case[param]
    assert_allclose(actual, expected, atol=0.0001, rtol=0.0005)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import gzip
import json
import os

import numpy as np
import pytest

from pygmm.model import Scenario
from pygmm import AkkarSandikkayaBommer2014 as ASB14

# Relative tolerance for all tests
RTOL = 1e-2

# Load the tests
fname = os.path.join(os.path.dirname(__file__), 'data', 'asb14_tests.json.gz')
with gzip.open(fname, 'rt') as fp:
    tests = json.load(fp)

testdata = [(dist, t['params'], results)
            for t in tests for dist, results in t['results']]


def create_model(params, dist):
    params = dict(params)
    params[dist] = params.pop('dist')
    s = Scenario(**params)
    m = ASB14(s)
    return m


@pytest.mark.parametrize('dist,params,expected', testdata)
def test_spec_accels(dist, params, expected):
    m = create_model(params, dist)
    np.testing.assert_allclose(
        m.interp_spec_accels(expected['periods']),
        expected['spec_accels'],
        rtol=RTOL,
        err_msg='Spectral accelerations')


@pytest.mark.parametrize('dist,params,expected', testdata)
@pytest.mark.parametrize('key', ['pga', 'pgv'])
def test_im_values(dist, params, expected, key):
    m = create_model(params, dist)
    np.testing.assert_allclose(
        getattr(m, key),
        expected[key],
        rtol=RTOL, )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Create plots for visual comparison."""

import os

import matplotlib
matplotlib.use('agg')  # NOQA
import matplotlib.pyplot as plt
import pytest

import pygmm

DEFAULT_PROPS = dict(
    depth_2_5=5,
    depth_bor=15,
    depth_hyp=9,
    depth_tor=5,
    dip=90.,
    dist=20,
    dist_jb=30.,
    dist_rup=30.,
    dist_x=30.,
    dpp_centered=0,
    flag_hw=0,
    flag_meas=0,
    mag=6,
    mechanism='SS',
    region='california',
    v_s30=500.,
    width=10, )

# Make the figure directory if needed
if not os.path.exists('figures'):
    os.makedirs('figures')


@pytest.mark.parametrize('model', pygmm.models, ids=lambda m: m.ABBREV)
@pytest.mark.parametrize(
    'key,values,label', [
        ('mag', [5, 6, 7], 'Magnitude'),
        (['dist', 'dist_rup', 'dist_jb', 'dist_x'], [10, 50, 100],
         'Distance (km)'),
        ('v_s30', [300, 650, 1000], '$V_{s30}$ (m/s)'),
    ],
    ids=lambda a: a[0])
def plot_model_with_param(model, key, values, label):
    props = dict(DEFAULT_PROPS)

    fig, ax = plt.subplots()
    for v in values:
        if isinstance(key, str):
            props[key] = v
        else:
            for k in key:
                props[k] = v
        m = model(**props)
        ax.plot(m.periods, m.spec_accels, label='%g' % v)

    ax.set_xlabel('Period (s)')
    try:
        ax.set_xscale('log')
    except ValueError:
        pass

    ax.set_ylabel('5% Damped, Spectral. Accel. (g)')
    ax.set_yscale('log')
    ax.set_ylim(1e-4, 1e1)

    ax.legend(loc='upper right', title=label, fontsize='x-small')

    ax.grid()

    fig.tight_layout()

    if isinstance(key, str):
        prefix = key
    else:
        prefix = key[0]

    fig.savefig(os.path.join('figures', prefix + '-' + model.ABBREV))
    plt.close(fig)



# -*- coding: utf-8 -*-
"""Akkar, Sandikkaya, & Bommer (2014, :cite:`akkar14`) model."""

import collections

import numpy as np

from . import model

__author__ = 'Albert Kottke'


class AkkarSandikkayaBommer2014(model.GroundMotionModel):
    """Akkar, Sandikkaya, & Bommer (2014, :cite:`akkar14`) model.

    The model is specified for three different distance metrics. However,
    the implementation uses only one distance metric. They are used in
    the following order:

        1. `dist_jb`

        2. `dist_hyp`

        3. `dist_epi`

    This order was selected based on evaluation of the total standard
    deviation. To compute the response for differing metrics, call the
    model multiple times with different keywords.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Akkar, Sandikkaya, & Bommer (2014)'
    ABBREV = 'ASB14'

    # Reference velocity (m/sec)
    V_REF = 750.

    # Load the coefficients for the model
    COEFF = collections.OrderedDict(
        (k, model.load_data_file('akkar-sandikkaya-bommer-2014-%s.csv' % k, 2))
        for k in ['dist_jb', 'dist_hyp', 'dist_epi'])
    PERIODS = np.array(COEFF['dist_jb'].period)

    INDICES_PSA = np.arange(2, 64)
    INDEX_PGA = 0
    INDEX_PGV = 1
    PARAMS = [
        model.NumericParameter('dist_jb', False, 0, 200),
        model.NumericParameter('dist_epi', False, 0, 200),
        model.NumericParameter('dist_hyp', False, 0, 200),
        model.NumericParameter('mag', True, 4, 8),
        model.NumericParameter('v_s30', True, 150, 1200),
        model.CategoricalParameter('mechanism', True, ['SS', 'NS', 'RS']),
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model."""
        super(AkkarSandikkayaBommer2014, self).__init__(scenario)

        s = self._scenario
        for k in self.COEFF:
            if s[k] is not None:
                dist = s[k]
                c = self.COEFF[k]
                break
        else:
            raise NotImplementedError(
                "Must provide at least one distance metric.")

        # Compute the reference response
        ln_resp_ref = (
            c.a_1 + c.a_3 * (8.5 - s.mag) ** 2 +
            (c.a_4 + c.a_5 *
             (s.mag - c.c_1)) * np.log(np.sqrt(dist ** 2 + c.a_6 ** 2)))
        mask = (s.mag <= c.c_1)
        ln_resp_ref[mask] += (c.a_2 * (s.mag - c.c_1))[mask]
        ln_resp_ref[~mask] += (c.a_7 * (s.mag - c.c_1))[~mask]

        if s.mechanism == 'NS':
            ln_resp_ref += c.a_8
        elif s.mechanism == 'RS':
            ln_resp_ref += c.a_9

        pga_ref = np.exp(ln_resp_ref[self.INDEX_PGA])

        # Compute the nonlinear site term
        if s.v_s30 <= self.V_REF:
            vs_ratio = s.v_s30 / self.V_REF
            site = (c.b_1 * np.log(vs_ratio) + c.b_2 * np.log(
                (pga_ref + c.c * vs_ratio ** c.n) / (
                    (pga_ref + c.c) * vs_ratio ** c.n)))
        else:
            site = c.b_1 * np.log(np.minimum(s.v_s30, c.v_con) / self.V_REF)

        self._ln_resp = ln_resp_ref + site
        self._ln_std = np.array(c.sd_total)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import gzip
import json
import os

import numpy as np
import pytest

from pygmm.model import Scenario
from pygmm import DerrasBardCotton2014 as DBC14

# Relative tolerance for all tests
RTOL = 1e-2

# Load the tests
fname = os.path.join(os.path.dirname(__file__), 'data', 'dbc14_tests.json.gz')
with gzip.open(fname, 'rt') as fp:
    tests = json.load(fp)

testdata = [(t['params'], t['results']) for t in tests]


@pytest.mark.parametrize('params,expected', testdata)
def test_spec_accels(params, expected):
    m = DBC14(Scenario(**params))
    np.testing.assert_allclose(
        m.interp_spec_accels(expected['periods']),
        # Need to convert from m/sec to g
        np.array(expected['spec_accels']) / m.GRAVITY,
        rtol=RTOL,
        err_msg='Spectral accelerations')


@pytest.mark.parametrize('params,expected', testdata)
@pytest.mark.parametrize('key', ['pga', 'pgv'])
def test_im_values(params, expected, key):
    m = DBC14(Scenario(**params))
    # PGA needs to be converted from m/sec² to g, and PGV need to be
    # converted from m/sec into cm/sec
    scale = m.GRAVITY if key == 'pga' else 0.01

    np.testing.assert_allclose(
        getattr(m, key),
        expected[key] / scale,
        rtol=RTOL, )


@pytest.fixture
def model():
    """Instance of the DBC13 model."""
    return DBC14(
        Scenario(dist_jb=10, mag=6, v_s30=600, depth_hyp=10, mechanism='SS'))


def test_ln_std(model):
    # Log10 total standard deviations from the paper
    expected = [
        0.298, 0.309, 0.31, 0.313, 0.319, 0.323, 0.325, 0.328, 0.335, 0.338,
        0.338, 0.337, 0.338, 0.337, 0.335, 0.334, 0.333, 0.33, 0.328, 0.328,
        0.326, 0.325, 0.322, 0.322, 0.326, 0.328, 0.329, 0.33, 0.33, 0.329,
        0.327, 0.328, 0.328, 0.327, 0.33, 0.331, 0.332, 0.332, 0.332, 0.331,
        0.33, 0.331, 0.333, 0.335, 0.339, 0.343, 0.346, 0.353, 0.356, 0.359,
        0.362, 0.365, 0.368, 0.368, 0.37, 0.37, 0.373, 0.375, 0.377, 0.378,
        0.378, 0.376, 0.375, 0.375
    ]

    np.testing.assert_allclose(
        np.log10(np.exp(model._ln_std)), expected, rtol=RTOL)



"""Abrahamson and Silva (1996, :cite:`abrahamson96`) duration model."""

from __future__ import division

import numpy as np

from . import model

__author__ = 'Albert Kottke'


class AbrahamsonSilva1996(model.Model):
    """Abrahamson and Silva (1996, :cite:`abrahamson96`) duration model.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Abrahamson Silva (1996)'
    ABBREV = 'AS96'

    PARAMS = [
        model.NumericParameter('mag', True, 4, 7.5),
        model.NumericParameter('dist_rup', True, 0, 250),
        # FIXME add site_cond to Scenario
        model.CategoricalParameter('site_cond', True, ['soil', 'rock']),
    ]

    def __init__(self, scenario):
        super(AbrahamsonSilva1996, self).__init__(scenario)

        s = self._scenario

        stress_drop = np.exp(5.204 + 0.851 * (s.mag - 6))
        moment = 10 ** (1.5 * s.mag + 16.05)

        self._ln_dur = np.log(
            (stress_drop / moment) ** (-1/3) / (4.9e6 * 3.2) +
            0.805 * (1 if s.site_cond == 'soil' else 0) +
            0.063 * max(s.dist_rup - 10, 0)
        ) + self.calc_ln_dur_incr(0.75)
        self._std_err = 0.55

    @property
    def duration(self):
        return np.exp(self._ln_dur)

    @property
    def std_err(self):
        return self._std_err

    @staticmethod
    def calc_ln_dur_incr(nias):
        # Mask out inappropriate values
        nias = np.asarray(nias)
        mask = (nias < 0.10) | (0.95 < nias)
        nias[mask] = np.nan

        # Compute the increment due to the difference in the normalized Arias
        # intensity
        ln_i_ratio = np.log((nias - 0.05) / (1 - nias))
        ln_dur_incr = -0.532 + 0.552 * ln_i_ratio - 0.0262 * ln_i_ratio ** 2

        return ln_dur_incr

    def interp(self, nias, stds=None):
        # Mask out inappropriate values
        nias = np.asarray(nias)
        mask = (nias < 0.10) | (0.95 < nias)
        nias[mask] = np.nan

        ln_dur = self._ln_dur + self.calc_ln_dur_incr(nias)

        if stds is not None:
            std_errs = np.interp(
                nias,
                [0.10, 0.15, 0.20, 0.25, 0.30, 0.35, 0.40, 0.45, 0.50, 0.55,
                 0.60, 0.65, 0.70, 0.75, 0.80, 0.85, 0.90, 0.95],
                [0.843, 0.759, 0.713, 0.691, 0.674, 0.660, 0.646, 0.636, 0.628,
                 0.616, 0.605, 0.594, 0.582, 0.565, 0.545, 0.528, 0.510, 0.493]
            )
            ln_dur = ln_dur + np.array(stds)[:, np.newaxis] * std_errs

        return np.exp(ln_dur)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Create test cases for the DBC14 model."""

import gzip
import json
import os

import xlwings as xw

params = {
    'depth_hyp': [10, 0, 25],
    'dist_jb': [5, 40, 200],
    'mag': [4, 6, 7],
    'mechanism': ['NS', 'RS', 'SS'],
    'v_s30': [200, 400, 800],
}


def iter_parameters(parameters):
    max_len = max(len(p) for p in parameters.values())

    for i in range(max_len):
        yield {k: v[i % len(v)] for k, v in parameters.items()}


def reformat_params(params):
    """Translate to spreadsheet values."""
    params['mechanism'] = 1 + ['NS', 'RS', 'SS'].index(params['mechanism'])
    return params


def load_params(wb, **params):
    cell_map = {
        'dist_jb': 'B08',
        'mag': 'B10',
        'v_s30': 'B12',
        'depth_hyp': 'B14',
        'mechanism': 'B16',
    }
    params = reformat_params(dict(params))
    for k, v in params.items():
        print('Setting:', k, cell_map[k], v)
        xw.Range('ANN(PGA)', cell_map[k], wkb=wb).value = v

    # Force a re-calculation. This is the same as Ctrl + Alt + F9
    wb.xl_app.CalculateFull()


def get_results(wb):
    d = {}
    # Collect PGA and PGV
    for key, row in zip(['pga', 'pgv'], [13, 10]):
        d[key] = xw.Range('ANN(PGA)', 'O%d' % row, wkb=wb).value

    # Collect response spectrum
    for key, rc in zip(['periods', 'spec_accels'], 'NO'):
        d[key] = xw.Range('ANN(PGA)', '%s17:%s78' % (rc, rc), wkb=wb).value

    return d


wb_fname = os.path.abspath('10518_2013_9481_MOESM1_ESM.xlsx')
tests = []
for p in iter_parameters(params):
    wb = xw.Workbook(wb_fname)
    load_params(wb, **p)
    tests.append({'params': p, 'results': get_results(wb)})

fname = '../tests/data/dbc14_tests.json.gz'
with gzip.open(fname, 'wt') as fp:
    json.dump(tests, fp, indent=4)



# -*- coding: utf-8 -*-
"""Basic models."""

import collections
import logging
import os

import numpy as np

from scipy.interpolate import interp1d

from typing import Optional, List
from .types import ArrayLike


class Scenario(collections.UserDict):
    r"""An eathquake scenario used in all ground motion models.

    Parameters
    ----------
    depth_1_0 : float
        depth to the 1.0 km∕s shear-wave velocity horizon beneath the site,
        :math:`Z_{1.0}` in (km).
    depth_2_5 : float
        depth to the 2.5 km∕s shear-wave velocity horizon beneath the site,
        :math:`Z_{2.5}` in (km).
    depth_tor : float
        depth to the top of the rupture plane (:math:`Z_{tor}`, km).
    depth_bor : float
        depth to the bottom of the rupture plane (:math:`Z_{bor}`, km).
    depth_bot : float
        depth to bottom of seismogenic crust (km).
    dip : float
        fault dip angle (:math:`\phi`, deg).
    dist_jb : float
        Joyner-Boore distance to the rupture plane (:math:`R_\text{JB}`, km)
    dist_crjb : float
        centroid Joyner-Boore distance, which is the shortest distance between
        the centroid of Joyner-Boore rupture surface of the potential Class 2
        earthquakes and the closest point on the edge of the Joyner-Boore
        rupture surface of the main shock (:math:`CR_\text{JB}`, km)
    dist_epi : float
        epicentral distance to the rupture plane (:math:`R_\text{epi}`, km)
    dist_hyp : float
        hypocentral distance to the rupture plane (:math:`R_\text{hyp}`, km).
    dist_rup : float
        closest distance to the rupture plane (:math:`R_\text{rup}`, km)
    dist_x : float
        site coordinate measured perpendicular to the fault strike from the
        fault line with the down-dip direction being positive (:math:`R_x`,
        km).
    dist_y0 : float
        horizontal distance off the end of the rupture measured parallel to
        strike (:math:`R_{y0}`, km).
    dpp_centered : float
        direct point parameter (DPP) for directivity effect (see Chiou and
        Spudich (2014, :cite:`spudich14`)) centered on the earthquake-specific
        average DPP for California.
    event_type : str
        event type. Type of event used in subduction models to distinguish
        between intraslab and interface events.
    is_aftershock : bool
        if the scenario is an aftershock.
    mag : float
        moment magnitude of the event (:math:`M_w`)
    mechanism : str
        fault mechanism. Valid options: "SS", "NS", "RS", and "U". See
        :ref:`Mechanism` for more information.
    on_hanging_wall : bool
        If the site is located on the hanging wall of the fault. If *None*,
        then *False* is assumed.
    region : str
        region. Valid options are specified in a specific GMM.
    site_cond : str
        site condition. String description of the site condition. Valid
        options are specified in a specific GMM.
    tectonic_region : str
        tectonic region. Tectonic setting of the site typically used in
        subductin models.
    v_s30 : float
        time-averaged shear-wave velocity over the top 30 m of the site
        (:math:`V_{s30}`, m/s).
    vs_source : str
        source of the `v_s30` value.  Valid options include: "measured",
        "inferred"
    width : float
        down-dip width of the fault.

    """

    KNOWN_KEYS = [
        'depth_1_0', 'depth_2_5', 'depth_tor', 'depth_bor', 'depth_bot',
        'depth_hyp', 'dip', 'dist_crjb', 'dist_jb', 'dist_epi', 'dist_hyp',
        'dist_rup', 'dist_x', 'dist_y0', 'dpp_centered', 'event_type',
        'is_aftershock', 'mag', 'mechanism', 'on_hanging_wall', 'region',
        'site_cond', 'tectonic_region', 'v_s30', 'vs_source', 'width'
    ]

    def __init__(self, **kwds):
        """Initialize the scenario."""
        super().__init__(kwds)
        for k in self.data:
            if k not in self.KNOWN_KEYS:
                raise Warning('%s is not a recognized scenario key!' % k)

    def __getattr__(self, item):
        """Access the data with attributes."""
        return self.data[item]

    def __repr__(self):
        """Representation."""
        return '<Scenario(mag={mag}, dist_jb={dist_jb})>'.format(**self.data)


class Model(object):
    #: Long name of the model
    NAME = ''
    #: Short name of the model
    ABBREV = ''

    def __init__(self, *args, **kwargs):
        """Initialize the model."""
        super(Model, self).__init__()

        self._ln_resp = None
        self._ln_std = None

        if len(args) == 1:
            scenario = args[0]
        else:
            scenario = Scenario(**kwargs)

        # Select the used parameters and check them against the recommended
        # values
        self._scenario = Scenario(
            **{p.name: scenario.get(p.name, None)
               for p in self.PARAMS})
        self._check_inputs()

    def _check_inputs(self):
        for p in self.PARAMS:
            self._scenario[p.name] = p.check(self._scenario[p.name])

    @property
    def scenario(self):
        return self._scenario


class GroundMotionModel(Model):
    """Abstract class for ground motion prediction models."""

    #: Indices for the spectral accelerations
    INDICES_PSA = np.array([])
    #: Indices of the periods
    PERIODS = np.array([])
    #: Index of the peak ground acceleration
    INDEX_PGA = None
    #: Index of the peak ground velocity
    INDEX_PGV = None
    #: Index of the peak ground displacement
    INDEX_PGD = None
    #: Limits of model applicability
    LIMITS = dict()
    #: Model parameters
    PARAMS = []
    #: Scale factor to apply to get PGV in cm/sec
    PGV_SCALE = 1.
    #: Scale factor to apply to get PGD in cm
    PGD_SCALE = 1.

    def __init__(self, scenario: Scenario):
        """Initialize the model."""
        super(GroundMotionModel, self).__init__(scenario)

        self._ln_resp = None
        self._ln_std = None

        # Select the used parameters and check them against the recommended
        # values
        self._scenario = Scenario(
            **{p.name: scenario.get(p.name, None)
               for p in self.PARAMS})
        self._check_inputs()

    def interp_spec_accels(self,
                           periods: ArrayLike,
                           kind: Optional[str] = 'linear') -> np.ndarray:
        """Interpolate the spectral acceleration.

        Interpolation of the spectral acceleration is done in natural log
        space.

        Parameters
        ----------
        periods : array_like
            spectral periods to interpolate the response.
        kind : str, optional
            see :func:`scipy.interpolate.interp1d` for description of kind.
            Options include: 'linear' (default), 'nearest', 'zero', 'slinear',
            'quadratic', and 'cubic'

        Returns
        -------
        spec_accels : np.ndarray
            interpolated spectral accelerations

        """
        return np.exp(
            interp1d(
                np.log(self.periods),
                self._ln_resp[self.INDICES_PSA],
                kind=kind,
                copy=False,
                bounds_error=False,
                fill_value=np.nan, )(np.log(periods)))

    def interp_ln_stds(self, periods: ArrayLike,
                       kind: Optional[str] = 'linear') -> np.ndarray:
        r"""Interpolate the logarithmic standard deviation.

        Interpolate the logarithmic standard deviation (:math:`\sigma_{\ln}`)
        of spectral acceleration at the provided damping at specified periods.

        Parameters
        ----------
        periods : array_like
            spectral periods to interpolate the response.
        kind : str, optional
            see :func:`scipy.interpolate.interp1d` for description of kind.
            Options include: 'linear' (default), 'nearest', 'zero', 'slinear',
            'quadratic', and 'cubic'

        Returns
        -------
        ln_stds : np.ndarray
            interpolated logarithmic standard deviations

        """
        if self._ln_std is None:
            raise NotImplementedError
        else:
            return interp1d(
                np.log(self.periods),
                self._ln_std[self.INDICES_PSA],
                kind=kind,
                copy=False,
                bounds_error=False,
                fill_value=np.nan, )(np.log(periods))

    @property
    def periods(self) -> np.ndarray:
        """Periods specified by the model."""
        return self.PERIODS[self.INDICES_PSA]

    @property
    def spec_accels(self) -> np.ndarray:
        """Pseudo-spectral accelerations computed by the model (g)."""
        return self._resp(self.INDICES_PSA)

    @property
    def ln_stds(self) -> np.ndarray:
        """Pseudo-spectral accelerations log-standard deviation."""
        if self._ln_std is None:
            raise NotImplementedError
        else:
            return self._ln_std[self.INDICES_PSA]

    @property
    def pga(self) -> float:
        """Peak ground acceleration (PGA) computed by the model (g)."""
        if self.INDEX_PGA is None:
            raise NotImplementedError
        else:
            return self._resp(self.INDEX_PGA)

    @property
    def ln_std_pga(self) -> float:
        """Peak ground accelaration log-standard deviation."""
        if self.INDEX_PGA is None:
            raise NotImplementedError
        else:
            return self._ln_std[self.INDEX_PGA]

    @property
    def pgv(self) -> float:
        """Peak ground velocity (PGV) computed by the model (cm/sec)."""
        if self.INDEX_PGV is None:
            raise NotImplementedError
        else:
            return self._resp(self.INDEX_PGV) * self.PGV_SCALE

    @property
    def ln_std_pgv(self) -> float:
        """Peak ground velocity log-standard deviation."""
        if self.INDEX_PGV is None:
            raise NotImplementedError
        else:
            return self._ln_std[self.INDEX_PGV]

    @property
    def pgd(self) -> float:
        """Peak ground displacement (PGD) computed by the model (cm)."""
        if self.INDEX_PGD is None:
            raise NotImplementedError
        else:
            return self._resp(self.INDEX_PGD) * self.PGD_SCALE

    @property
    def ln_std_pgd(self) -> float:
        """Peak ground displacement log-standard deviation."""
        if self.INDEX_PGD is None:
            raise NotImplementedError
        else:
            return self._ln_std[self.INDEX_PGD]

    def _resp(self, index) -> np.ndarray:
        if index is not None:
            return np.exp(self._ln_resp[index])

    def _check_inputs(self) -> None:
        for p in self.PARAMS:
            self._scenario[p.name] = p.check(self._scenario[p.name])

class Parameter(object):
    """Model parameter.

    Parameters
    ----------
    name : str
        parameter name
    required : bool
        if the parameter is required
    default : None
        (optional) default value. Use *None* for no default value.

    """

    def __init__(self, name, required=False, default=None):
        """Initialize the parameter."""
        super(Parameter, self).__init__()
        self._name = name
        self._required = required
        self._default = default

    def check(self, value):
        """Check the value against the limits."""
        if value is None and self.required:
            raise ValueError(self.name, 'is a required parameter')

        if value is None:
            value = self.default
        return value

    @property
    def default(self):
        """Value to use as default."""
        return self._default

    @property
    def name(self):
        """Parameter name."""
        return self._name

    @property
    def required(self):
        """If the parameter is required."""
        return self._required


class NumericParameter(Parameter):
    """Numeric parameter.

    Parameters
    ----------
    name : str
        parameter name
    required : bool
        if the parameter is required
    default : float or int
        (optional) default value. Use *None* for no default value.

    """

    def __init__(self,
                 name: str,
                 required: bool=False,
                 min_: Optional[float]=None,
                 max_: Optional[float]=None,
                 default: Optional[float]=None):
        """Initialize parameter."""
        super(NumericParameter, self).__init__(name, required, default)
        self._min = min_
        self._max = max_

    @property
    def min(self) -> float:
        """Minimum value."""
        return self._min

    @property
    def max(self) -> float:
        """Maximum value."""
        return self._max

    def check(self, value) -> float:
        """Check the value against the limits."""
        value = super(NumericParameter, self).check(value)
        if value is not None:
            if self.min is not None and value < self.min:
                logging.warning(
                    '%s (%g) is less than the recommended limit (%g).',
                    self.name, value, self.min)
            elif self.max is not None and self.max < value:
                logging.warning(
                    '%s (%g) is greater than the recommended limit (%g).',
                    self.name, value, self.max)

        return value


class CategoricalParameter(Parameter):
    """Categorical parameter.

    Parameters
    ----------
    name : str
        parameter name
    required : bool
        if the parameter is required
    options : list[str]
        list of options
    default : str
        (optional) default option. Use *None* for no default value.

    """

    def __init__(self,
                 name: str,
                 required: bool=False,
                 options: Optional[List[str]]=None,
                 default: Optional[str]=None):
        """Initialize parameter."""
        super(CategoricalParameter, self).__init__(name, required, default)
        self._options = options or []

    @property
    def options(self) -> List[str]:
        """Possible options."""
        return self._options

    def check(self, value) -> str:
        """Check the value against the limits."""
        value = super(CategoricalParameter, self).check(value)
        if value not in self.options:
            alert = logging.error if self.required else logging.warning
            alert('%s value of "%s" is not one of the options. The following'
                  ' options are possible: %s', self.name, value,
                  ', '.join([str(o) for o in self._options]))

            if not self.required:
                logging.warning('Using default value for %s', self.name)
                value = self.default

        return value


def load_data_file(name, skip_header=None) -> np.recarray:
    """Load a data file.

    Returns
    -------
    data : :class:`numpy.recarray`
       data values

    """
    fname = os.path.join(os.path.dirname(__file__), 'data', name)
    return np.recfromcsv(
        fname, skip_header=skip_header, case_sensitive=True).view(np.recarray)



# -*- coding: utf-8 -*-
"""Kishida (2017, :cite:`kishida17`) conditional spectrum."""

import numpy as np

from .baker_jayaram_2008 import calc_correls
from .types import ArrayLike


def calc_cond_mean_spectrum_vector(
        periods: ArrayLike,
        ln_psas: ArrayLike,
        ln_stds: ArrayLike,
        ln_psas_cond: ArrayLike) -> (np.ndarray, np.ndarray):
    """Kishida (2017, :cite:`kishida17`) conditional spectrum.

    Conditional mean spectrum vector (CMSV) by Kishida (2017,
    :cite:`kishida17`) is specifying the target spectral acceleration at
    multiple periods, rather than the single conditioning period by Cornell and
    Baker (2008). If this approach is used for a single period, then the
    resulting spectrum is the same as computed by Cornell and Baker (2008) --
    implemented by :func:`~pygmm.baker_jayaram_2008.calc_cond_mean_spectrum`.

    Parameters
    ----------
    periods : array_like
        Spectral periods of the response spectrum [sec]. This array must be
        increasing.
    ln_psas : array_like
        Natural logarithm of the spectral acceleration. Same length as
        `periods`.
    ln_stds : array_like
        Logarithmic standard deviation of the spectral acceleration. Same
        length as `periods`.
    ln_psas_cond : :class:`np.ma.masked_array`
        The vector of conditioning spectral accelerations. This is a masked
        array with the same length as `periods`. Masked values are not used
        for defining the CMSV.

    Returns
    -------
    ln_psas_cmsv : :class:`np.ndarray`
        Natural logarithm of the conditional mean spectral accelerations.
    ln_stds_cmsv : :class:`np.ndarray`
        Logarithmic standard deviation of the conditional mean spectral
        acceleration.

    Raises
    ------
    ValueError
        If `periods` are monotonically increasing.

    """
    periods = np.asarray(periods)
    ln_psas = np.asarray(ln_psas)
    ln_stds = np.asarray(ln_stds)

    if not np.all(np.diff(periods) > 0):
        raise ValueError("`periods` must be increasing")

    mask = ln_psas_cond.mask

    # Group the periods into those not being conditioned upon, and those used
    # for the conditioning. See Equations (14), (15), and (16)
    periods_grouped = np.r_[periods[mask], periods[~mask]]
    # Standard deviation matrix. Named V^{1/2} in Kishida, Equation (8)
    mat_ln_std = np.diag(np.r_[ln_stds[mask], ln_stds[~mask]])
    # Correlation matrix, Equation (9)
    mat_correls = np.vstack([
        calc_correls(periods_grouped, period_cond)
        for period_cond in periods_grouped
    ]).T
    # Covariance matrix, Equation (7)
    mat_covar = mat_ln_std @ mat_correls @ mat_ln_std
    # Extract the submatrices, Equation (6)
    n = np.ma.count_masked(ln_psas_cond)
    mat_covar_11 = mat_covar[0:n, 0:n]
    mat_covar_22 = mat_covar[n:, n:]
    mat_covar_12 = mat_covar[0:n, n:]
    mat_covar_21 = mat_covar[n:, 0:n]
    mat_covar_22_inv = np.linalg.inv(mat_covar_22)
    # Conditional mean value, Equation (3)
    mat_total_resids = (ln_psas_cond[~mask] - ln_psas[~mask]).T
    ln_psas_cmsv = np.r_[ln_psas[mask] + np.ravel(
        mat_covar_12 @ mat_covar_22_inv @ mat_total_resids), ln_psas_cond[
            ~mask].data]
    # Compute standard deviation, Equation (4)
    ln_stds_cmsv = np.r_[np.sqrt(
        np.diag(mat_covar_11 - (mat_covar_12 @ mat_covar_22_inv @ mat_covar_21
                                ))), np.zeros(ln_psas_cond.count())]
    # Sort to the original period indices
    indices = np.argsort(periods_grouped)
    ln_psas_cmsv = ln_psas_cmsv[indices]
    ln_stds_cmsv = ln_stds_cmsv[indices]

    return ln_psas_cmsv, ln_stds_cmsv



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Plot influence of V_s30 predicted by CY14 model."""

import matplotlib.pyplot as plt
import pygmm

fig, ax = plt.subplots()

for v_s30 in [300, 600, 900]:
    s = pygmm.model.Scenario(
        mag=7, dist_jb=20, dist_x=20, dist_rup=25, dip=90, v_s30=v_s30)
    m = pygmm.ChiouYoungs2014(s)
    ax.plot(m.periods, m.spec_accels, label=str(v_s30))

ax.set_xlabel('Period (s)')
ax.set_xscale('log')

ax.set_ylabel('5%-Damped Spectral Accel. (g)')
ax.set_yscale('log')

ax.grid()

ax.legend(title='$V_{s30}$ (m/s)')

plt.show()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file: history = history_file.read()
setup(
    name='pygmm',
    version='0.6.0',
    description="Ground motion models implemented in Python.",
    long_description=readme + '\n\n' + history,
    author="Albert Kottke",
    author_email='albert.kottke@gmail.com',
    url='https://github.com/arkottke/pygmm',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'matplotlib',
        'numpy',
        'scipy >= 0.17.0',
        'six',
    ],
    license='MIT',
    zip_safe=False,
    keywords='pygmm',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering',
        'Intended Audience :: Science/Research',
    ],
    setup_requires=[
        'pytest-runner',
    ],
    tests_require=[
        'pytest >= 2.9.0',
        'pytest-cov',
        'pytest-flake8',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Move the predictor into a arrays for usage."""

import os

import numpy as np

periods = [
    (-1, 'pgv'),
    (0, 'pga'),
    (0.1, 'T0.1'),
    (0.5, 'T0.5'),
    (1.0, 'T1'),
    (4.0, 'T4'),
]

path = './HermkesAl'

events = np.genfromtxt(
    os.path.join(path, 'predictors.csv'),
    delimiter=',',
    skip_header=1,
    # Do not create records as it complicates using the data in the
    # interpolation
    # names=['mag', 'depth_hyp', 'flag_rs', 'flag_ss', 'flag_ns', 'dist_jb',
    # 'v_s30'],
)

fname = 'gpselin_corrnoise_pred.csv'

predictions = []
for period, period_name in periods:
    p = np.genfromtxt(
        os.path.join(path, period_name, fname),
        delimiter=',', )
    predictions.append(p)

# Convert to a recarray
# names = [pn + suffix for p, pn in periods for suffix in ['', '_var']]

predictions = np.array([p[:, i] for p in predictions for i in range(2)]).T

np.savez_compressed(
    '../pygmm/data/hermkes_kuehn_riggelsen_2014.npz',
    events=events,
    predictions=predictions)

# Print test cases
for i in [0, 9470, 1245080, -1]:
    print('Event', i, '\n', events[i])
    print('Prediction', '\n', predictions[i])



# -*- coding: utf-8 -*-
"""Derras, Bard and Cotton (2014, :cite:`derras14`) model."""

import json
import os

import numpy as np

from . import model

__author__ = 'Albert Kottke'


class DerrasBardCotton2014(model.GroundMotionModel):
    """Derras, Bard and Cotton (2014, :cite:`derras14`) model.

    Parameters
    ----------
    scenario : :class:`pygmm.model.Scenario`
        earthquake scenario

    """

    NAME = 'Derras, Bard & Cotton (2014)'
    ABBREV = 'DBC13'

    # Load the coefficients for the model
    COEFF = json.load(
        open(
            os.path.join(
                os.path.dirname(__file__), 'data',
                'derras_bard_cotton_2014.json')))
    GRAVITY = 9.80665
    PERIODS = np.array(COEFF['period'])

    INDICES_PSA = np.arange(2, 64)
    INDEX_PGA = 1
    INDEX_PGV = 0
    PARAMS = [
        model.NumericParameter('dist_jb', True, 5, 200),
        model.NumericParameter('mag', True, 4, 7),
        model.NumericParameter('v_s30', True, 200, 800),
        model.NumericParameter('depth_hyp', True, 0, 25),
        model.CategoricalParameter('mechanism', True, ['SS', 'NS', 'RS']),
    ]

    def __init__(self, scenario: model.Scenario):
        """Initialize the model."""
        super().__init__(scenario)
        c = self.COEFF
        # Values modified during the calculation
        s = dict(self._scenario)

        for k in ['v_s30', 'dist_jb']:
            s['log10_' + k] = np.log10(s[k])
        # Translate to mechanism integer
        s['mechanism'] = dict(NS=1, RS=3, SS=4)[s['mechanism']]

        # Create the normalized parameter matrix
        keys = [
            'log10_dist_jb', 'mag', 'log10_v_s30', 'depth_hyp', 'mechanism'
        ]
        values = np.array([s[k] for k in keys])
        limits = np.rec.array([c['min_max'][k] for k in keys], names='min,max')
        p_n = np.array(2 * (values - limits['min']) / (limits['max'] - limits[
            'min']) - 1).T

        # Compute the normalized response
        b_1 = np.array(c['b_1']).T
        b_2 = np.array(c['b_2']).T
        w_1 = np.array(c['w_1'])
        w_2 = np.array(c['w_2'])

        log10_resp_n = (b_2 + w_2 @ np.tanh(b_1 + w_1 @ p_n))

        # Convert from normalized values
        log10_resp_limits = np.rec.array(
            c['min_max']['log10_resp'], names='min,max')
        log10_resp = (0.5 * (log10_resp_n + 1) *
                      (log10_resp_limits['max'] - log10_resp_limits['min']
                       ) + log10_resp_limits['min'])
        # Convert from m/sec and m/sec² into cm/sec and g
        scale = np.log10(
            np.r_[0.01, self.GRAVITY * np.ones(self.PERIODS.size - 1)])

        self._ln_resp = np.log(10 ** (log10_resp - scale))
        self._ln_std = np.log(10 ** np.array(c['log10_std']['total']))
